@@ -5,7 +5,7 @@
 # 修改记录：
 #   2022-05-12  WXG  verify date格式 支持 参数 no-hms, 时分秒为 00:00:00
 #   2022-06-05  WXG  verify in参数增加缺少结束符 ']' 的校验。版本 1.1.3
-#   2022-07-08  WXG  verify 增加ppt，自定义错误提示 prompt。版本 1.1.4
+#   2022-07-08  WXG  verify 增加ppt，自定义错误提示。版本 1.1.4
 # =============================================================================
 from collections import defaultdict
 from abc import ABCMeta, abstractmethod
@@ -14,15 +14,9 @@ from ..tools import JdTools, JdDate
 from .errcode import *
 
 
-ERR_VERIFY_NEED_PARAM = 500     # 缺少参数
-ERR_VERIFY_VAL_EXCEED = 501     # 数值越限
-ERR_VERIFY_ERROR = 900          # 校验出错
-ERR_VERIFY_WHO = 901            # 未知的校验类型
-
-
-__version__ = "1.1.6"
+__version__ = "1.1.7"
 __author__ = "WXG"
-__all__ = ['verify', 'validation', 'ERR_VERIFY_NEED_PARAM', 'ERR_VERIFY_VAL_EXCEED', 'ERR_VERIFY_ERROR']
+__all__ = ['verify', 'validation']
 
 
 class Validator(object):
@@ -215,8 +209,34 @@ class Any(OpValidator):
 
 class Prompt(OpValidator):
     """提示语，代替默认提示"""
-    def __init__(self, prompt):
-        self.prompt = prompt
+    def __init__(self, ppt):
+        self.ppt = ppt
+
+
+class _WRONG(object):
+    pass
+
+
+def _set_default(default, who):
+    """
+    计算默认值， ds
+        ds 默认值， str 默认值为 空字符串("")， date/time -- None,
+        decimal/int -- 0, dateRange -- 不支持
+    :param default:     默认值， s for ds， 0/1/5.6 for int, decimal: d0, d5.6
+    :param who:         参数类型：int, date, str, ...
+    :return:    默认值 or _WRONG
+    """
+    dd = default
+    if default in ["s", "0"]:  # 默认值 ds
+        if who in ['int', 'i', 'D', 'decimal']:
+            dd = 0
+        elif who in ['s', 'str']:
+            dd = ""
+        elif who in ['t', 'dateRange']:
+            dd = _WRONG
+        elif who in ['d', 'date', 't', 'time']:     # time 也支持 ds
+            dd = None
+    return dd
 
 
 def validation(s, key, who):
@@ -259,7 +279,13 @@ def validation(s, key, who):
             validations.append(NoHms)
         elif i[0] == 'd':  # default，默认值
             val = i[1:]
-            if '.' in val:
+            success = True
+            if val in ['s', '0']:  # ds - 默认值
+                default = _set_default(val, who)
+                if default == _WRONG:
+                    errors[key].append("time/dateRange不支持默认值！")
+                    success = False
+            elif '.' in val:
                 success, default = JdTools.decimal(val)
                 if not success:
                     errors[key].append("默认值不合法！")
@@ -367,9 +393,12 @@ def verify(obj, **kwargs):
         优先级  opt > must > d(default)
         1. 【opt】 为 可选参数， 没有对应 key 时， 写库不传递该 key ； 有对应
            key 时，opt 不起作用。
-           不加 opt，按必填参数处理。
+           - 优先级 opt < default(d)  即有默认值时，opt 无效
+           - 不加 opt，按必填参数处理 ★★★。
         2. 【must】 为 必填参数。 没有对应 key 时 报错。 日期时， None 值报错
         3. 【d】 为 默认值(default)， 未传递 key 或者 值为 '', None 时，赋默认值。
+            - ds 默认值， str 默认值为 空字符串("")， date -- None,
+              decimal/int -- 0, time / dateRange -- 不支持
         4. scope 取值范围包括： >=0(int, decimal 默认范围), >0, >x, >=x;
            any (任意取值)。
         5. 【p】point, 默认为 p2 小数点后2位，只对 decimal 有效 。
@@ -415,6 +444,8 @@ def verify(obj, **kwargs):
         if k == 'tip':
             extra_tip = v or ''  # 附加提示信息，放在开头。 比如 对续班子表校验时，可输入 学员姓名
         else:
+            if not isinstance(v, dict):  # 异常处理
+                return {'code': ERR_SYNTAX, 'msg': ERR_SYNTAX_MSG % f"{k}={v}"}
             for k2, v2 in v.items():    # 遍历 date， int 等 具体类型的校验
                 ret = _validate_single_key(k2, v2, obj, k, extra_tip)
                 if ret['code'] != 0:
@@ -434,6 +465,28 @@ def _validate_single_key(key, s, dictionary, who, extra_tip):
     name = f'{extra_tip}“{tip}”' if extra_tip else f'{tip}'
     if errors:
         return {'code': ERR_VERIFY_ERROR, 'msg': f'{extra_tip}“{tip}”，{errors[key][0]}'}
+
+    # 读取特殊 校验 对象
+    special = {'ppt': {'has': False, 'val': ''},        # ppt， 提示
+               'default': {'has': False, 'val': ''},    # 默认值
+               'point': {'has': False, 'val': ''},      # 小数点后位数
+               'add_hms': {'has': True},                # 是否添加 时分秒 for date
+               }
+    for validator in validations:
+        if isinstance(validator, Prompt):  # 自定义提示
+            special['ppt'].update({'has': True, 'val': validator.ppt})
+        elif isinstance(validator, Default):  # 默认值
+            special['default'].update({'has': True, 'val': validator.default})
+        elif isinstance(validator, Point):  # 小数点
+            special['point'].update({'has': True, 'val': validator.point})
+        elif validator == NoHms:  # 不需要添加 年月日
+            special['add_hms'].update({'has': False})
+
+    # 默认值
+    if key not in dictionary or dictionary[key] == '' or dictionary[key] is None:
+        if special['default']['has']:
+            dictionary[key] = special['default']['val']
+
     if Optional in validations and key not in dictionary:  # 可选参数
         return {'code': 0, 'msg': 'ok'}
     if Required in validations and key not in dictionary:  # 必填参数
@@ -446,29 +499,8 @@ def _validate_single_key(key, s, dictionary, who, extra_tip):
                 if dictionary[key] is None or dictionary[key] == '':
                     return {'code': ERR_VERIFY_NEED_PARAM, 'msg': f'请输入【{name}】'}
 
-    # 读取特殊 校验 对象
-    special = {'prompt': {'has': False, 'val': ''},
-               'default': {'has': False, 'val': ''},
-               'point': {'has': False, 'val': ''},
-               'add_hms': {'has': True},
-               }
-    for validator in validations:
-        if isinstance(validator, Prompt):   # 自定义提示
-            special['prompt'].update({'has': True, 'val': validator.prompt})
-        elif isinstance(validator, Default):    # 默认值
-            special['default'].update({'has': True, 'val': validator.default})
-        elif isinstance(validator, Point):   # 小数点
-            special['point'].update({'has': True, 'val': validator.point})
-        elif validator == NoHms:    # 不需要添加 年月日
-            special['add_hms'].update({'has': False})
-
-    # 默认值
-    if key not in dictionary or dictionary[key] == '' or dictionary[key] is None:
-        if special['default']['has']:
-            dictionary[key] = special['default']['val']
-
     if key not in dictionary:
-        return {'code': E_PUB_NEED, 'msg': E_PUB_NEED_MSG % name}
+        return {'code': ERR_PUB_NEED, 'msg': special['ppt']['val'] or ERR_PUB_NEED_MSG % name}
 
     val = dictionary[key]
     if who in ['s', 'str']:
@@ -480,31 +512,28 @@ def _validate_single_key(key, s, dictionary, who, extra_tip):
 
         passed, dictionary[key] = JdTools.decimal_check(val, pt)
         if not passed:
-            return {"code": E_PUB_NUM, "msg": E_PUB_NUM_MSG % name}
+            return {"code": ERR_PUB_NUM, "msg": special['ppt']['val'] or ERR_PUB_NUM_MSG % name}
         dec = JdTools.decimal(val, pt=pt + 1)
         if dictionary[key] != dec:
-            return {"code": E_PUB_NUM_PT2, "msg": f'{name} 最多 {pt} 位小数！'}
+            return {"code": ERR_PUB_NUM_PT2, "msg": f'{name} 最多 {pt} 位小数！'}
         # 校验 decimal 数值 不能超过 7 位数字
         if dec > 9999999:
-            return {'code': ERR_VERIFY_VAL_EXCEED, 'msg': f'{name} 数值越限，最多7位（除小数外）'}
+            return {'code': ERR_VERIFY_VAL_EXCEED, 'msg': special['ppt']['val'] or f'{name} 数值越限，最多7位（除小数外）'}
     elif who in ['d', 'date']:
         add_hms = special['add_hms']['has']
         try:
             dictionary[key] = JdTools.str_to_datetime(val, add_hms=add_hms)
         except (ValueError, TypeError):
-            return {'code': ERR_DATE_FMT, 'msg': f'{name} 格式错误【{val}】，期望 年-月-日！'}
+            return {'code': ERR_DATE_FMT, 'msg': special['ppt']['val'] or f'{name} 格式错误【{val}】，期望 年-月-日！'}
     elif who in ['t', 'time']:      # 时间 时分秒
         try:
             dictionary[key] = JdTools.str_to_time(val)
         except (ValueError, TypeError):
-            return {'code': 201, 'msg': f'{name} 格式错误【{val}】，期望 HH:MM:SS！'}
+            return {'code': ERR_TIME_FMT, 'msg': special['ppt']['val'] or f'{name} 格式错误【{val}】，期望 HH:MM:SS！'}
     elif who in ['i', 'int']:
         passed, dictionary[key] = JdTools.int(val)
         if not passed:
-            if special['prompt']['has']:
-                return {"code": E_PUB_CUSTOMIZED_ERR, "msg": special['prompt']['val']}
-            else:
-                return {"code": E_PUB_NUM_INT, "msg": E_PUB_NUM_INT_MSG % name}
+            return {"code": ERR_PUB_NUM_INT, "msg": special['ppt']['val'] or ERR_PUB_NUM_INT_MSG % name}
     elif who in ['dateRange']:
         ret = JdDate.start_end_time('time_range', val)
         if ret['code'] != 0:
@@ -520,9 +549,6 @@ def _validate_single_key(key, s, dictionary, who, extra_tip):
             continue
         passed = validator(dictionary[key])
         if not passed:
-            if special['prompt']['has']:
-                return {"code": E_PUB_CUSTOMIZED_ERR, "msg": special['prompt']['val']}
-            else:
-                return {'code': 900, 'msg': f'[{name}] {validator.err_message}'}
+            return {'code': ERR_VERIFY_ERROR, 'msg': special['ppt']['val'] or f'[{name}] {validator.err_message}'}
 
     return {'code': 0, 'msg': 'ok'}
