@@ -18,6 +18,7 @@
 #   val_list = cfg.get('section_name')     # val_list 为 [('key', 1), ('key2', 'test')]
 #
 # ConfigParser 会将所有的 key 都转换为 小写
+# 注释字符 为 英文 分号 ; 。 井号 不再作为注释使用
 # =============================================================================
 import os
 
@@ -26,7 +27,7 @@ from threading import Lock
 from .tools import JdTools
 
 
-__version__ = '1.3'
+__version__ = '1.4'
 
 
 def lock(func):
@@ -96,19 +97,11 @@ class JdIniReader(object):
 
     @staticmethod
     def __trim_del_comment(val):
-        # 过滤空格， 删除 ; 或 # 号及后面的所有注释字符
+        # 过滤空格， 删除 ; 号及后面的所有注释字符
+        # 其中井号(#) 不再作为注释使用，因为 密码中常用#，容易截断密码。使用转义就复杂了
         i = val.find(';')
-        j = val.find('#')
-        if i >= 0 and j >= 0:
-            m = min(i, j)
-        elif i >= 0:
-            m = i
-        elif j >= 0:
-            m = j
-        else:
-            m = -1  # 分号和井号都没有
-        if m >= 0:
-            val = val[0: m]
+        if i >= 0:
+            val = val[0: i]
         v2 = JdTools.trim(val)
         try:
             v2 = int(v2)    # 若可以转换为 int，则转换，否则原样返回
